@@ -402,6 +402,7 @@ struct parse_key {
 	{"KEY_MEDIA_REPEAT", 0x1b7},
 	{"KEY_10CHANNELSUP", 0x1b8},
 	{"KEY_10CHANNELSDOWN", 0x1b9},
+	{"KEY_IMAGES", 0x1ba},
 	{"KEY_DEL_EOL", 0x1c0},
 	{"KEY_DEL_EOS", 0x1c1},
 	{"KEY_INS_LINE", 0x1c2},
@@ -454,6 +455,12 @@ struct parse_key {
 	{"KEY_TOUCHPAD_TOGGLE", 0x212},
 	{"KEY_TOUCHPAD_ON", 0x213},
 	{"KEY_TOUCHPAD_OFF", 0x214},
+	{"KEY_CAMERA_ZOOMIN", 0x215},
+	{"KEY_CAMERA_ZOOMOUT", 0x216},
+	{"KEY_CAMERA_UP", 0x217},
+	{"KEY_CAMERA_DOWN", 0x218},
+	{"KEY_CAMERA_LEFT", 0x219},
+	{"KEY_CAMERA_RIGHT", 0x21a},
 	{"BTN_TRIGGER_HAPPY", 0x2c0},
 	{"BTN_TRIGGER_HAPPY1", 0x2c0},
 	{"BTN_TRIGGER_HAPPY2", 0x2c1},
