@@ -40,6 +40,23 @@
 
 #define CTRL_FLAG_DISABLED (V4L2_CTRL_FLAG_READ_ONLY | V4L2_CTRL_FLAG_INACTIVE | V4L2_CTRL_FLAG_GRABBED)
 
+static bool is_valid_type(enum v4l2_ctrl_type type)
+{
+	switch (type) {
+	case V4L2_CTRL_TYPE_INTEGER:
+	case V4L2_CTRL_TYPE_BOOLEAN:
+	case V4L2_CTRL_TYPE_MENU:
+	case V4L2_CTRL_TYPE_BUTTON:
+	case V4L2_CTRL_TYPE_INTEGER64:
+	case V4L2_CTRL_TYPE_BITMASK:
+	case V4L2_CTRL_TYPE_CTRL_CLASS:
+	case V4L2_CTRL_TYPE_STRING:
+		return true;
+	default:
+		return false;
+	}
+}
+
 void ApplicationWindow::addWidget(QGridLayout *grid, QWidget *w, Qt::Alignment align)
 {
 	grid->addWidget(w, m_row, m_col, align | Qt::AlignVCenter);
@@ -60,7 +77,8 @@ void ApplicationWindow::addTabs()
 	memset(&qctrl, 0, sizeof(qctrl));
 	qctrl.id = V4L2_CTRL_FLAG_NEXT_CTRL;
 	while (queryctrl(qctrl)) {
-		if ((qctrl.flags & V4L2_CTRL_FLAG_DISABLED) == 0) {
+		if (is_valid_type(qctrl.type) &&
+		    (qctrl.flags & V4L2_CTRL_FLAG_DISABLED) == 0) {
 			m_ctrlMap[qctrl.id] = qctrl;
 			if (qctrl.type != V4L2_CTRL_TYPE_CTRL_CLASS)
 				m_classMap[V4L2_CTRL_ID2CLASS(qctrl.id)].push_back(qctrl.id);
@@ -76,6 +94,8 @@ void ApplicationWindow::addTabs()
 			qctrl.id = id;
 			if (!queryctrl(qctrl))
 				continue;
+			if (!is_valid_type(qctrl.type))
+				continue;
 			if (qctrl.flags & V4L2_CTRL_FLAG_DISABLED)
 				continue;
 			m_ctrlMap[qctrl.id] = qctrl;
@@ -83,6 +103,8 @@ void ApplicationWindow::addTabs()
 		}
 		for (qctrl.id = V4L2_CID_PRIVATE_BASE;
 				queryctrl(qctrl); qctrl.id++) {
+			if (!is_valid_type(qctrl.type))
+				continue;
 			if (qctrl.flags & V4L2_CTRL_FLAG_DISABLED)
 				continue;
 			m_ctrlMap[qctrl.id] = qctrl;
@@ -95,7 +117,8 @@ void ApplicationWindow::addTabs()
 		unsigned id = m_classMap[V4L2_CTRL_CLASS_USER][i];
 
 		if (m_ctrlMap[id].type == V4L2_CTRL_TYPE_INTEGER64 ||
-		    m_ctrlMap[id].type == V4L2_CTRL_TYPE_STRING) {
+		    m_ctrlMap[id].type == V4L2_CTRL_TYPE_STRING ||
+		    V4L2_CTRL_DRIVER_PRIV(id)) {
 			m_haveExtendedUserCtrls = true;
 			break;
 		}
@@ -238,6 +261,18 @@ void ApplicationWindow::addCtrl(QGridLayout *grid, const v4l2_queryctrl &qctrl)
 				m_sigMapper, SLOT(map()));
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		addLabel(grid, name);
+		edit = new QLineEdit(p);
+		edit->setInputMask("HHHHHHHH");
+		addWidget(grid, edit);
+		m_widgetMap[qctrl.id] = edit;
+		connect(m_widgetMap[qctrl.id], SIGNAL(lostFocus()),
+				m_sigMapper, SLOT(map()));
+		connect(m_widgetMap[qctrl.id], SIGNAL(returnPressed()),
+				m_sigMapper, SLOT(map()));
+		break;
+
 	case V4L2_CTRL_TYPE_STRING:
 		addLabel(grid, name);
 		edit = new QLineEdit(p);
@@ -433,6 +468,9 @@ int ApplicationWindow::getVal(unsigned id)
 		v = static_cast<QCheckBox *>(w)->isChecked();
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		v = (int)static_cast<QLineEdit *>(w)->text().toUInt(0, 16);
+		break;
 	case V4L2_CTRL_TYPE_MENU:
 		idx = static_cast<QComboBox *>(w)->currentIndex();
 		for (i = qctrl.minimum; i <= qctrl.maximum; i++) {
@@ -514,14 +552,17 @@ void ApplicationWindow::refresh(unsigned ctrl_class)
 	if (!m_haveExtendedUserCtrls && ctrl_class == V4L2_CTRL_CLASS_USER) {
 		for (unsigned i = 0; i < m_classMap[ctrl_class].size(); i++) {
 			unsigned id = m_classMap[ctrl_class][i];
-
 			v4l2_control c;
 
+			queryctrl(m_ctrlMap[id]);
+			if (m_ctrlMap[id].type == V4L2_CTRL_TYPE_BUTTON)
+				continue;
+			if (m_ctrlMap[id].flags & V4L2_CTRL_FLAG_WRITE_ONLY)
+				continue;
 			c.id = id;
 			if (ioctl(VIDIOC_G_CTRL, &c)) {
 				errorCtrl(id, errno);
 			}
-			queryctrl(m_ctrlMap[id]);
 			setVal(id, c.value);
 			m_widgetMap[id]->setDisabled(m_ctrlMap[id].flags & CTRL_FLAG_DISABLED);
 		}
@@ -622,6 +663,16 @@ void ApplicationWindow::setWhat(QWidget *w, unsigned id, long long v)
 		w->setStatusTip(w->whatsThis());
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		w->setWhatsThis(QString("Type: Bitmask\n"
+					"Maximum: %1\n"
+					"Current: %2\n"
+					"Default: %3")
+			.arg((unsigned)qctrl.maximum, 0, 16).arg((unsigned)v, 0, 16)
+			.arg((unsigned)qctrl.default_value, 0, 16) + flags);
+		w->setStatusTip(w->whatsThis());
+		break;
+
 	case V4L2_CTRL_TYPE_BUTTON:
 		w->setWhatsThis(QString("Type: Button") + flags);
 		w->setStatusTip(w->whatsThis());
@@ -666,6 +717,10 @@ void ApplicationWindow::setVal(unsigned id, int v)
 			static_cast<QLineEdit *>(w)->setText(QString::number(v));
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		static_cast<QLineEdit *>(w)->setText(QString("%1").arg((unsigned)v, 8, 16, QChar('0')));
+		break;
+
 	case V4L2_CTRL_TYPE_BOOLEAN:
 		static_cast<QCheckBox *>(w)->setChecked(v);
 		break;
@@ -729,7 +784,7 @@ void ApplicationWindow::setDefaults(unsigned ctrl_class)
 		if (m_ctrlMap[id].type == V4L2_CTRL_TYPE_INTEGER64)
 			setVal64(id, 0);
 		else if (m_ctrlMap[id].type == V4L2_CTRL_TYPE_STRING)
-			setString(id, "");
+			setString(id, QString(' ', m_ctrlMap[id].minimum));
 		else if (m_ctrlMap[id].type != V4L2_CTRL_TYPE_BUTTON)
 			setVal(id, m_ctrlMap[id].default_value);
 	}
